@@ -6,9 +6,12 @@ module Debian.Debianize.BuildDependencies
     ) where
 
 
+#if !MIN_VERSION_base(4,8,0)
 import Control.Applicative ((<$>))
+#endif
 import Control.Lens
 import Control.Monad.State (MonadState(get))
+import Control.Monad.Trans (liftIO, MonadIO)
 import Data.Char (isSpace, toLower)
 import Data.Function (on)
 import Data.List as List (filter, groupBy, map, minimumBy, nub, sortBy)
@@ -16,8 +19,8 @@ import Data.Map as Map (lookup, Map)
 import Data.Maybe (catMaybes, fromMaybe, isJust, isNothing, listToMaybe)
 import Data.Set as Set (empty, fold, fromList, map, member, Set, singleton, toList, union)
 import Data.Version (showVersion, Version)
-import Debian.Debianize.BasicInfo (buildEnv, compilerFlavor, EnvSet(dependOS), verbosity)
-import Debian.Debianize.Bundled (builtIn, hcVersion)
+import Debian.Debianize.BasicInfo (buildEnv, compilerFlavor, EnvSet(dependOS))
+import Debian.Debianize.Bundled (builtIn)
 import qualified Debian.Debianize.DebInfo as D
 import Debian.Debianize.DebianName (mkPkgName, mkPkgName')
 import Debian.Debianize.Monad as Monad (CabalInfo, CabalT)
@@ -30,13 +33,12 @@ import Debian.Orphans ()
 import Debian.Relation (BinPkgName(..), checkVersionReq, Relation(..), Relations)
 import qualified Debian.Relation as D (BinPkgName(BinPkgName), Relation(..), Relations, VersionReq(EEQ, GRE, LTE, SGR, SLT))
 import Debian.Version (DebianVersion, parseDebianVersion')
-import Debug.Trace (trace)
 import Distribution.Compiler (CompilerFlavor(..))
 import Distribution.Package (Dependency(..), PackageIdentifier(pkgName, pkgVersion), PackageName(PackageName))
 import Distribution.PackageDescription (PackageDescription)
 import Distribution.PackageDescription as Cabal (BuildInfo(..), BuildInfo(buildTools, extraLibs, pkgconfigDepends), Library(..), Executable(..), TestSuite(..))
 import qualified Distribution.PackageDescription as Cabal (PackageDescription(library, executables, testSuites))
-import Distribution.Text (display)
+--import Distribution.Text (display)
 import Distribution.Version (anyVersion, asVersionIntervals, earlierVersion, foldVersionRange', fromVersionIntervals, intersectVersionRanges, isNoVersion, laterVersion, orEarlierVersion, orLaterVersion, toVersionIntervals, unionVersionRanges, VersionRange, withinVersion)
 import Distribution.Version.Invert (invertVersionRange)
 import Prelude hiding (init, log, map, unlines, unlines, writeFile)
@@ -105,10 +107,10 @@ mergeCabalDependencies =
 
 -- The haskell-cdbs package contains the hlibrary.mk file with
 -- the rules for building haskell packages.
-debianBuildDeps :: (Monad m, Functor m) => PackageDescription -> CabalT m D.Relations
+debianBuildDeps :: (MonadIO m, Functor m) => PackageDescription -> CabalT m D.Relations
 debianBuildDeps pkgDesc =
-    do hc <- use (A.debInfo . D.flags . compilerFlavor)
-       let hcs = singleton hc -- vestigial
+    do hflavor <- use (A.debInfo . D.flags . compilerFlavor)
+       let hcs = singleton hflavor -- vestigial
        let hcTypePairsLibs =
                fold union empty $
                   Set.map (\ hc' -> Set.map (hc',) $ hcPackageTypesLibs hc') hcs
@@ -135,15 +137,20 @@ debianBuildDeps pkgDesc =
        prof <- not <$> use (A.debInfo . D.noProfilingLibrary)
        official <- use (A.debInfo . D.official)
        compat <- use (A.debInfo . D.compat)
+       let ghcdev = compilerPackageName hflavor B.Development
+       let ghcrel = if member GHC hcs then maybe [] ((: []) . anyrel') ghcdev else []
+       let ghcprof = compilerPackageName hflavor B.Profiling
+       let ghcrelprof = if prof then maybe [] ((: []) . anyrel') ghcprof else []
        let xs = nub $ [maybe [] (\ n -> [D.Rel (D.BinPkgName "debhelper") (Just (D.GRE (parseDebianVersion' (show n)))) Nothing]) compat,
-                       [D.Rel (D.BinPkgName "haskell-devscripts") (Just $ D.GRE $ parseDebianVersion' $ if official then "0.9" else "0.8" :: String) Nothing],
+                       [D.Rel (D.BinPkgName "haskell-devscripts-minimal") Nothing Nothing,
+                        D.Rel (D.BinPkgName "haskell-devscripts") (Just $ D.GRE $ parseDebianVersion' $ if official then "0.9" else "0.8" :: String) Nothing],
                        anyrel "cdbs"] ++
-                      (if member GHC hcs
-                       then [anyrel' (compilerPackageName GHC B.Development)] ++ if prof then [anyrel' (compilerPackageName GHC B.Profiling)] else []
-                       else []) ++
+                      (ghcrel ++ ghcrelprof) ++
+{-
 #if MIN_VERSION_Cabal(1,22,0)
-                      (if member GHCJS hcs then [anyrel "ghcjs"] else []) ++
+                      (if member GHCJS (Set.map _hcFlavor hcs) then [anyrel (compilerPackageName hflavor B.Development)] else []) ++
 #endif
+-}
                        bDeps ++
                        cDeps
        filterMissing xs
@@ -163,8 +170,9 @@ debianBuildDeps pkgDesc =
       hcPackageTypesTests :: CompilerFlavor -> Set B.PackageType
       hcPackageTypesTests _ = singleton B.Development
 
-
-debianBuildDepsIndep :: (Monad m, Functor m) => PackageDescription -> CabalT m D.Relations
+-- | Collect the dependencies required to build any packages that have
+-- architecture "all".
+debianBuildDepsIndep :: (MonadIO m, Functor m) => PackageDescription -> CabalT m D.Relations
 debianBuildDepsIndep pkgDesc =
     do hc <- use (A.debInfo . D.flags . compilerFlavor)
        let hcs = singleton hc -- vestigial
@@ -172,19 +180,17 @@ debianBuildDepsIndep pkgDesc =
        bDeps <- use (A.debInfo . D.control . S.buildDependsIndep)
        libDeps <- allBuildDepends (maybe [] (return . libBuildInfo) (Cabal.library pkgDesc))
        cDeps <- mapM docDependencies libDeps
+       let ghcdoc = compilerPackageName hc B.Documentation
+       let hcdocdep = if doc && member GHC hcs then maybe [] ((: []) . anyrel') ghcdoc else []
        let xs = nub $ if doc && isJust (Cabal.library pkgDesc)
-                      then (if member GHC hcs then [anyrel' (compilerPackageName GHC B.Documentation)] else []) ++
-#if MIN_VERSION_Cabal(1,22,0)
-                           (if member GHCJS hcs then [anyrel "ghcjs"] else []) ++
-#endif
-                           bDeps ++ concat cDeps
+                      then hcdocdep ++ bDeps ++ concat cDeps
                       else []
        filterMissing xs
 
 -- | The documentation dependencies for a package include the
 -- documentation package for any libraries which are build
 -- dependencies, so we have use to all the cross references.
-docDependencies :: (Monad m, Functor m) => Dependency_ -> CabalT m D.Relations
+docDependencies :: (MonadIO m, Functor m) => Dependency_ -> CabalT m D.Relations
 docDependencies (BuildDepends (Dependency name ranges)) =
     do hc <- use (A.debInfo . D.flags . compilerFlavor)
        let hcs = singleton hc -- vestigial
@@ -195,7 +201,7 @@ docDependencies _ = return []
 -- | The Debian build dependencies for a package include the profiling
 -- libraries and the documentation packages, used for creating cross
 -- references.  Also the packages associated with extra libraries.
-buildDependencies :: (Monad m, Functor m) => Set (CompilerFlavor, B.PackageType) -> Dependency_ -> CabalT m D.Relations
+buildDependencies :: (MonadIO m, Functor m) => Set (CompilerFlavor, B.PackageType) -> Dependency_ -> CabalT m D.Relations
 buildDependencies hcTypePairs (BuildDepends (Dependency name ranges)) =
     use (A.debInfo . D.omitProfVersionDeps) >>= \ omitProfDeps ->
     concat <$> mapM (\ (hc, typ) -> dependencies hc typ name ranges omitProfDeps) (toList hcTypePairs)
@@ -250,7 +256,7 @@ anyrel' x = [D.Rel x Nothing Nothing]
 -- | Turn a cabal dependency into debian dependencies.  The result
 -- needs to correspond to a single debian package to be installed,
 -- so we will return just an OrRelation.
-dependencies :: Monad m => CompilerFlavor -> B.PackageType -> PackageName -> VersionRange -> Bool -> CabalT m Relations
+dependencies :: MonadIO m => CompilerFlavor -> B.PackageType -> PackageName -> VersionRange -> Bool -> CabalT m Relations
 dependencies hc typ name cabalRange omitProfVersionDeps =
     do nameMap <- use A.debianNameMap
        -- Compute a list of alternative debian dependencies for
@@ -317,35 +323,28 @@ dependencies hc typ name cabalRange omitProfVersionDeps =
 -- compiler a substitute for that package.  If we were to
 -- specify the virtual package (e.g. libghc-base-dev) we would
 -- have to make sure not to specify a version number.
-doBundled :: Monad m =>
+doBundled :: MonadIO m =>
              B.PackageType
           -> PackageName
           -> CompilerFlavor
           -> [D.Relation]
           -> CabalT m [D.Relation]
-doBundled typ name hc rels =
-    mapM doRel rels >>= return . concat
+doBundled typ name hc rels = do
+  let hcname = compilerPackageName hc typ
+  mapM (doRel hcname) rels >>= return . concat
     where
       -- If a library is built into the compiler, this is the debian
       -- package name the compiler will conflict with.
-      comp = D.Rel (compilerPackageName hc typ) Nothing Nothing
-      doRel :: Monad m => D.Relation -> CabalT m [D.Relation]
-      doRel rel@(D.Rel dname req _) = do
+      doRel :: Monad m => Maybe BinPkgName -> D.Relation -> CabalT m [D.Relation]
+      doRel hcname rel@(D.Rel dname req _) = do
+        let comp = maybe [] (\x -> [D.Rel x Nothing Nothing]) hcname
         -- gver <- use ghcVersion
         root <- use (A.debInfo . D.flags . buildEnv) >>= return . dependOS
         -- Look at what version of the package is provided by the compiler.
         atoms <- get
         -- What version of this package (if any) does the compiler provide?
-        let Just hcv = hcVersion root hc
-            relInfo = builtIn hc root
+        let relInfo = builtIn hc root
             pver = listToMaybe $ fmap (debianVersion'' atoms) (filter ((== name) . pkgName) relInfo)
-            v = view (A.debInfo . D.flags . verbosity) atoms
-        case (v > 0, filter ((== name) . pkgName) relInfo) of
-          (True, [i]) ->
-              trace ("  " ++ show hc ++ "-" ++ show (showVersion hcv) ++ " in " ++ root ++ " provides " ++ display (pkgName i) ++ "-" ++ display (pkgVersion i)) (return ())
-          (True, _) ->
-              trace ("  " ++ show hc ++ "-" ++ show (showVersion hcv) ++ " in " ++ root ++ " does not provide " ++ display name) (return ())
-          _ -> return ()
         -- The name this library would have if it was in the compiler conflicts list.
         let naiveDebianName = mkPkgName hc name typ
         -- The compiler should appear in the build dependency
@@ -354,7 +353,7 @@ doBundled typ name hc rels =
         -- library (which, if pver is Nothing, will certainly
         -- result in an error which needs to be corrected in
         -- the packaging.)
-        let compilerDependency = if isJust pver && (checkVersionReq req pver || (dname == naiveDebianName && conflictsWithHC naiveDebianName)) then [comp] else []
+        let compilerDependency = if isJust pver && (checkVersionReq req pver || (dname == naiveDebianName && conflictsWithHC naiveDebianName)) then comp else []
         -- The library package can satisfy the dependency if
         -- the compiler doesn't provide a version, or if the
         -- compiler doesn't conflict with the package's
