@@ -1,5 +1,5 @@
 -- | How to name the debian packages based on the cabal package name and version number.
-{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, OverloadedStrings, RankNTypes, ScopedTypeVariables, StandaloneDeriving, TypeFamilies #-}
+{-# LANGUAGE CPP, FlexibleInstances, MultiParamTypeClasses, OverloadedStrings, RankNTypes, ScopedTypeVariables, StandaloneDeriving, TypeFamilies #-}
 {-# OPTIONS -Wall -Wwarn -fno-warn-name-shadowing -fno-warn-orphans #-}
 module Debian.Debianize.DebianName
     ( debianName
@@ -12,7 +12,9 @@ module Debian.Debianize.DebianName
     ) where
 
 
+#if !MIN_VERSION_base(4,8,0)
 import Control.Applicative ((<$>))
+#endif
 import Control.Lens
 import Data.Char (toLower)
 import Data.Map as Map (alter, lookup)
@@ -40,13 +42,13 @@ data Dependency_
 
 -- | Build the Debian package name for a given package type.
 debianName :: (Monad m, Functor m, PkgName name) => PackageType -> CompilerFlavor -> CabalT m name
-debianName typ cfl =
+debianName typ hc =
     do base <-
-           case (typ, cfl) of
+           case (typ, hc) of
              (Utilities, GHC) -> use (debInfo . utilsPackageNameBase) >>= maybe (((\ base -> "haskell-" ++ base ++ "-utils") . unDebBase) <$> debianNameBase) return
              (Utilities, _) -> use (debInfo . utilsPackageNameBase) >>= maybe (((\ base -> base ++ "-utils") . unDebBase) <$> debianNameBase) return
              _ -> unDebBase <$> debianNameBase
-       return $ mkPkgName' cfl typ (DebBase base)
+       return $ mkPkgName' hc typ (DebBase base)
 
 -- | Function that applies the mapping from cabal names to debian
 -- names based on version numbers.  If a version split happens at v,
@@ -70,23 +72,23 @@ debianNameBase =
 -- correspondence between the PackageType value and the name type, so
 -- it can return nonsense like (SrcPkgName "libghc-debian-dev").
 mkPkgName :: PkgName name => CompilerFlavor -> PackageName -> PackageType -> name
-mkPkgName cfl pkg typ = mkPkgName' cfl typ (debianBaseName pkg)
+mkPkgName hc pkg typ = mkPkgName' hc typ (debianBaseName pkg)
 
 mkPkgName' :: PkgName name => CompilerFlavor -> PackageType -> DebBase -> name
-mkPkgName' cfl typ (DebBase base) =
+mkPkgName' hc typ (DebBase base) =
     pkgNameFromString $
              case typ of
                 Documentation -> prefix ++ base ++ "-doc"
                 Development -> prefix ++ base ++ "-dev"
                 Profiling -> prefix ++ base ++ "-prof"
-                Utilities -> base {- ++ case cfl of
+                Utilities -> base {- ++ case hc of
                                           GHC -> ""
-                                          _ -> "-" ++ map toLower (show cfl) -}
+                                          _ -> "-" ++ map toLower (show hc) -}
                 Exec -> base
                 Source -> base
                 HaskellSource -> "haskell-" ++ base
                 Cabal -> base
-    where prefix = "lib" ++ map toLower (show cfl) ++ "-"
+    where prefix = "lib" ++ map toLower (show hc) ++ "-"
 
 debianBaseName :: PackageName -> DebBase
 debianBaseName (PackageName name) =
