@@ -6,7 +6,10 @@ module Debian.Debianize.Finalize
     ) where
 
 
+#if !MIN_VERSION_base(4,8,0)
 import Control.Applicative ((<$>))
+import Data.Monoid (mempty)
+#endif
 import Control.Lens hiding ((<.>))
 import Control.Monad (unless, when)
 import Control.Monad as List (mapM_)
@@ -19,7 +22,7 @@ import Data.Function (on)
 import Data.List as List (filter, intercalate, map, nub, null, unlines, maximumBy)
 import Data.Map as Map (delete, elems, insertWith, lookup, Map, toList)
 import Data.Maybe (fromMaybe, isJust, fromJust)
-import Data.Monoid ((<>), mempty)
+import Data.Monoid ((<>))
 import Data.Set as Set (difference, filter, fold, fromList, insert, map, null, Set, singleton, toList, union, unions)
 import Data.Set.Extra as Set (mapM_)
 import Data.Text as Text (intercalate, pack, Text, unlines, unpack)
@@ -37,6 +40,7 @@ import Debian.Debianize.Monad as Monad (CabalT, liftCabal)
 import Debian.Debianize.Prelude ((.?=))
 import qualified Debian.Debianize.SourceDebDescription as S
 import Debian.Debianize.VersionSplits (DebBase(DebBase))
+import Debian.GHC (compilerPackageName)
 import Debian.Orphans ()
 import Debian.Policy (getCurrentDebianUser, getDebhelperCompatLevel, haskellMaintainer, maintainerOfLastResort, PackageArchitectures(Any, All), PackagePriority(Extra), parseMaintainer, parseStandardsVersion, Section(..), SourceFormat(Native3))
 import Debian.Pretty (PP(..), ppShow)
@@ -90,7 +94,7 @@ finalizeDebianization =
 -- this function is not idempotent.  (Exported for use in unit tests.)
 -- FIXME: we should be able to run this without a PackageDescription, change
 --        paramter type to Maybe PackageDescription and propagate down thru code
-finalizeDebianization'  :: (Monad m, Functor m) => String -> Maybe NameAddr -> Maybe Int -> Bool -> CabalT m ()
+finalizeDebianization'  :: (MonadIO m, Functor m) => String -> Maybe NameAddr -> Maybe Int -> Bool -> CabalT m ()
 finalizeDebianization' date currentUser debhelperCompat setupExists =
     do -- In reality, hcs must be a singleton or many things won't work.  But some day...
        hc <- use (A.debInfo . D.flags . compilerFlavor)
@@ -102,10 +106,9 @@ finalizeDebianization' date currentUser debhelperCompat setupExists =
          (True, D.TestsRun) -> (A.debInfo . D.rulesSettings) %= (++ ["DEB_ENABLE_TESTS = yes"])
          (True, D.TestsBuild) -> (A.debInfo . D.rulesSettings) %= (++ ["DEB_ENABLE_TESTS = yes", "DEB_BUILD_OPTIONS += nocheck"])
          _ -> return ()
-       case setupExists of
-         False -> (A.debInfo . D.rulesSettings) %= (++ ["DEB_SETUP_BIN_NAME = cabal"])
-         _ -> return ()
-
+       (A.debInfo . D.rulesSettings) %=
+          (++ ["DEB_SETUP_BIN_NAME = " <> if setupExists then "debian/hlibrary.setup" else "cabal"])
+ 
        finalizeSourceName B.HaskellSource
        checkOfficialSettings hc
        addExtraLibDependencies hc
@@ -211,10 +214,18 @@ debianEpoch name = get >>= return . Map.lookup name . view A.epochMap
 finalizeSourceName :: (Monad m, Functor m) => B.PackageType -> CabalT m ()
 finalizeSourceName typ =
     do DebBase debName <- debianNameBase
-       (A.debInfo . D.sourcePackageName) .?= Just (SrcPkgName (case typ of
-                                                   B.HaskellSource -> "haskell-" ++ debName
-                                                   B.Source -> debName
-                                                   _ -> error $ "finalizeSourceName: " ++ show typ))
+       hc <- use (A.debInfo . D.flags . compilerFlavor)
+       (A.debInfo . D.sourcePackageName) .?=
+          Just (SrcPkgName (case (hc, typ) of
+                              -- Haskell source deb names conventionally have the prefix
+                              -- "haskell-" added.  Here we add prefix "ghcjs-" to
+                              -- haskell packages build with the ghcjs compiler.
+                              (GHC, B.HaskellSource) -> "haskell-" ++ debName
+#if MIN_VERSION_Cabal(1,22,0)
+                              (GHCJS, B.HaskellSource) -> "ghcjs-" ++ debName
+#endif
+                              (_, B.Source) -> debName
+                              _ -> error $ "finalizeSourceName: " ++ show typ))
 
 -- | Try to compute a string for the the debian "Maintainer:" and
 -- "Uploaders:" fields using, in this order
@@ -369,7 +380,7 @@ officialSettings = do
               , S.VCSGit  "https://anonscm.debian.org/cgit/pkg-haskell/DHG_packages.git"
               ])
 
-putBuildDeps :: (Monad m, Functor m) => (Relations -> Relations) -> PackageDescription -> CabalT m ()
+putBuildDeps :: (MonadIO m, Functor m) => (Relations -> Relations) -> PackageDescription -> CabalT m ()
 putBuildDeps finalizeRelations pkgDesc =
     do deps <- debianBuildDeps pkgDesc >>= return . finalizeRelations
        depsIndep <- debianBuildDepsIndep pkgDesc >>= return . finalizeRelations
@@ -650,13 +661,14 @@ expandAtoms =
              List.mapM_ (\ (b, f) -> modify (execAtoms b f)) (Map.toList mp)
 
 -- | Add the normal default values to the rules files.
-finalizeRules :: (Monad m, Functor m) => CabalT m ()
+finalizeRules :: (MonadIO m, Functor m) => CabalT m ()
 finalizeRules =
     do DebBase b <- debianNameBase
-       compiler <- use (A.debInfo . D.flags . compilerFlavor)
+       hc <- use (A.debInfo . D.flags . compilerFlavor)
+       let BinPkgName hcdeb = maybe (error "No compiler package") id (compilerPackageName hc B.Development)
        (A.debInfo . D.rulesHead) .?= Just "#!/usr/bin/make -f"
        (A.debInfo . D.rulesSettings) %= (++ ["DEB_CABAL_PACKAGE = " <> pack b])
-       (A.debInfo . D.rulesSettings) %= (++ (["DEB_DEFAULT_COMPILER = " <> pack (List.map toLower (show compiler))]))
+       (A.debInfo . D.rulesSettings) %= (++ ["DEB_DEFAULT_COMPILER = " <> pack hcdeb])
        flags <- (flagString . Set.toList) <$> use (A.debInfo . D.flags . cabalFlagAssignments)
        unless (List.null flags) ((A.debInfo . D.rulesSettings) %= (++ ["DEB_SETUP_GHC6_CONFIGURE_ARGS = " <> pack flags]))
        (A.debInfo . D.rulesIncludes) %= (++ ["include /usr/share/cdbs/1/rules/debhelper.mk",
