@@ -19,6 +19,9 @@ import Data.Char (isSpace)
 import Data.Map as Map (insert, insertWith)
 import Data.Maybe (fromMaybe)
 import Data.Monoid ((<>))
+#if !MIN_VERSION_base(4,8,0)
+import Data.Monoid (mappend)
+#endif
 import Data.Set as Set (fromList, insert, singleton)
 import Data.Text (break, lines, null, pack, strip, Text, unpack, words)
 import Data.Text.IO (readFile)
@@ -184,8 +187,8 @@ loadChangeLog =
     doPaths ["CHANGELOG", "ChangeLog", "changelog", "debian/changelog"]
     where
       doPaths :: [FilePath] -> IO (Maybe ChangeLog)
-      doPaths (p : ps) = doPath p >>= maybe (doPaths ps) (\log -> putStrLn ("Found valid changelog at " ++ p) >> return (Just log))
-      doPaths [] = pure Nothing
+      doPaths (p : ps) = doPath p >>= maybe (doPaths ps) (\log -> {-putStrLn ("Found valid changelog at " ++ p) >>-} return (Just log))
+      doPaths [] = return Nothing
       doPath :: FilePath -> IO (Maybe ChangeLog)
       doPath p = do
         t <- tryIOError (readFile p)
@@ -193,9 +196,9 @@ loadChangeLog =
           where
             doParse :: Text -> IO (Maybe ChangeLog)
             doParse t = do
-              pure $ either (const Nothing) Just (parseChangeLog (unpack t))
+              return $ either (const Nothing) Just (parseChangeLog (unpack t))
             doExn :: IOError -> IO (Maybe ChangeLog)
-            doExn e | isDoesNotExistError e = pure Nothing
+            doExn e | isDoesNotExistError e = return Nothing
             doExn e = error ("inputChangelog: " ++ show e)
 
 inputCabalInfoFromDirectory :: MonadIO m => DebianT m () -- .install files, .init files, etc.
