@@ -36,7 +36,7 @@ deriving instance Typeable CompilerId
 #if MIN_VERSION_Cabal(1,22,0)
 deriving instance Typeable AbiTag
 deriving instance Data AbiTag
-#if 1
+#if !MIN_VERSION_Cabal(1,24,0)
 deriving instance Eq AbiTag
 #endif
 deriving instance Ord AbiTag
@@ -46,7 +46,7 @@ deriving instance Data Compiler
 deriving instance Data CompilerId
 
 deriving instance Ord Language
-#if 1
+#if !MIN_VERSION_Cabal(1,24,0)
 deriving instance Eq Compiler
 #endif
 deriving instance Ord Compiler
